#include "Builder/Builders/SimpleBuilder.h"
#include "MazeContent/Buildings/Wall.h"
#include "MazeContent/Buildings/Pass.h"

SimpleBuilder::SimpleBuilder(float width, float height, float room_size):
                            CommonBuilder(width, height, room_size) {}

void SimpleBuilder::create_rooms() {
    size_t rows = static_cast<size_t>(m_height / m_room_size) % 2 == 0 ? m_height / m_room_size - 1 : m_height / m_room_size;
    size_t cols = static_cast<size_t>(m_width / m_room_size) % 2 == 0 ? m_width / m_room_size - 1 : m_width / m_room_size;
    sf::Vector2f top_left_origin_pos{ (m_width - cols * m_room_size) / 2 + m_room_size / 2,
                               (m_height - rows * m_room_size) / 2 + m_room_size / 2 };

    for(size_t i = 0; i < rows; ++i) {
        std::vector<std::unique_ptr<Room>> row;
        for (size_t j = 0; j < cols; ++j) {
            row.emplace_back(std::make_unique<Room>(m_room_size));
            row.at(j)->set_position(sf::Vector2f{top_left_origin_pos.x + j * m_room_size,
                                                 top_left_origin_pos.y + i * m_room_size});
        }
        m_rooms.emplace_back(std::move(row));
    }
}

void SimpleBuilder::set_rooms_sides() {
    for (size_t i = 0; i < m_rooms.size(); ++i) {
        for (size_t j = 0; j < m_rooms.begin()->size(); ++j) {
            if (i == 0) {
                auto up_wall = std::make_shared<Wall>(*m_rooms[i][j]);
                m_rooms[i][j]->set_side(Room::UP, up_wall);
                m_rooms[i][j]->get_side(Room::UP)->prepare_for_drawing();
            } else {
                auto pass = std::make_shared<Pass>(*m_rooms[i - 1][j], *m_rooms[i][j]);
                m_rooms[i - 1][j]->set_side(Room::DOWN, pass);
                m_rooms[i - 1][j]->get_side(Room::DOWN)->prepare_for_drawing();
                m_rooms[i][j]->set_side(Room::UP, pass);
                m_rooms[i][j]->get_side(Room::UP)->prepare_for_drawing();
            }
            if (j == 0) {
                auto left_wall = std::make_shared<Wall>(*m_rooms[i][j]);
                m_rooms[i][j]->set_side(Room::LEFT, left_wall);
                m_rooms[i][j]->get_side(Room::LEFT)->prepare_for_drawing();
            } else {
                auto pass = std::make_shared<Pass>(*m_rooms[i][j - 1], *m_rooms[i][j]);
                m_rooms[i][j - 1]->set_side(Room::RIGHT, pass);
                m_rooms[i][j - 1]->get_side(Room::RIGHT)->prepare_for_drawing();
                m_rooms[i][j]->set_side(Room::LEFT, pass);
                m_rooms[i][j]->get_side(Room::LEFT)->prepare_for_drawing();
            }
            if (i == m_rooms.size() - 1) {
                auto down_wall = std::make_shared<Wall>(*m_rooms[i][j]);
                m_rooms[i][j]->set_side(Room::DOWN, down_wall);
                m_rooms[i][j]->get_side(Room::DOWN)->prepare_for_drawing();
            }
            if (j == m_rooms.begin()->size() - 1) {
                auto right_wall = std::make_shared<Wall>(*m_rooms[i][j]);
                m_rooms[i][j]->set_side(Room::RIGHT, right_wall);
                m_rooms[i][j]->get_side(Room::RIGHT)->prepare_for_drawing();
            }
        }
    }
}
