#include "Draw/Button.h"
#include "Draw/MyFont.h"

Button::Button() {
    m_text.setFont(MyFont::Instance().get_font());
    m_text.setCharacterSize(config::BUTTON_FONT_SIZE);
    m_text.setFillColor(config::BUTTON_COLOR_TEXT);
    m_rectangle.setFillColor(config::BUTTON_COLOR_FILL);
    m_rectangle.setOutlineThickness(config::BUTTON_FRAME_THICKNESS);
    m_rectangle.setOutlineColor(config::BUTTON_COLOR_FRAME);
}

void Button::set(const sf::Vector2f& pos, const sf::Vector2f& button_size, const std::string& text,
                 size_t font_size, std::unique_ptr<ISelectCommand> ptr_command) {
    m_rectangle.setSize(button_size);
    m_rectangle.setOrigin(m_rectangle.getSize() / 2.f);
    m_rectangle.setPosition(pos);
    m_text.setString(text);
    auto bounds = m_text.getLocalBounds();
    m_text.setOrigin(bounds.left + bounds.width / 2.0f, bounds.top + bounds.height / 2.0f);
    m_text.setPosition(pos);
    m_ptr_command = std::move(ptr_command);
}

void Button::push() const {
    m_ptr_command->execute();
}

void Button::select() noexcept {
    m_rectangle.setFillColor(config::BUTTON_COLOR_SELECTION);
    m_is_selected = true;
}

void Button::unselect() noexcept {
    m_rectangle.setFillColor(config::BUTTON_COLOR_FILL);
    m_is_selected = false;
}

bool Button::is_selected() noexcept{
    return m_is_selected;
}

bool Button::is_position_in(sf::Vector2f pos) const noexcept {
    return m_rectangle.getGlobalBounds().contains(pos);
}

void Button::draw_into(sf::RenderWindow& window) const {
    window.draw(m_rectangle);
    window.draw(m_text);
}