#include "MazeContent/Entities/DynamicEntities/Enemy.h"
#include "Config.h"
#include <Events/IGameEvent.h>

Enemy::Enemy() : m_rectangle({ config::GAME_ENEMY_SIZE / 2, config::GAME_ENEMY_SIZE / 2 }) {
    m_rectangle.setOrigin(m_rectangle.getSize() / 2.f);
    m_rectangle.setFillColor(config::GAME_ENEMY_COLOR);
}

std::unique_ptr<IDynamicEntity> Enemy::clone() const {
    return std::make_unique<Enemy>(*this);
}

void Enemy::prepare_for_drawing() {
    m_rectangle.setPosition(m_ptr_room->get_position());
}

void Enemy::draw_into(sf::RenderWindow& window) const {
    window.draw(m_rectangle);
}

void Enemy::action() {
    auto miliseconds = static_cast<size_t>(m_stopwatch.getElapsedTime().asMilliseconds());
    if (miliseconds < rand() % 10000)
        return;
    auto direction = static_cast<Room::Direction>(rand() % 4);
    m_ptr_room->get_side(direction)->enter(this);
    m_stopwatch.restart();
}

std::unique_ptr<IGameEvent> Enemy::accept(IVisitor* ptr_visitor) {
    return ptr_visitor->visit(this);
}