#include "MazeContent/Entities/Pacman.h"
#include <Config.h>
#include "Events/DeleteStaticEntity.hpp"
#include "Events/LostGame.hpp"
#include "MazeContent/Entities/StaticEntities/Food.h"
#include "MazeContent/Entities/DynamicEntities/Enemy.h"

Pacman::Pacman() :m_circle(config::GAME_PACMAN_SIZE / 2) {
    m_circle.setOrigin(m_circle.getRadius(), m_circle.getRadius());
    m_circle.setFillColor(config::GAME_COLOR_PACMAN);
}

void Pacman::prepare_for_drawing() {
    m_circle.setPosition(m_ptr_room->get_position());
}

void Pacman::draw_into(sf::RenderWindow& window) const {
    window.draw(m_circle);
}

void Pacman::move(Room::Direction direction) {
    m_ptr_room->get_side(direction)->enter(this);
}

std::unique_ptr<IGameEvent> Pacman::visit(Food* ptr_food) {
    if (ptr_food->get_location() != this->get_location()) {
        return {};
    }
    return std::make_unique<DeleteStaticEntity>(ptr_food);
}

std::unique_ptr<IGameEvent> Pacman::visit(Enemy* ptr_enemy) {
    if (ptr_enemy->get_location() != this->get_location()) {
        return {};
    }
    return std::make_unique<LostGame>();
}